import os

class DefaultConfiguration(object):
    # Debugging purposes
    DEBUG = False
    BOOTSTRAP_SERVE_LOCAL = True

    # The line below is where to put the uploaded files.
    # Use the full path and do not use wildcards.
    # ~ is user's home directory
    UPLOAD_DIR = os.path.expanduser("~/Uploads")

    # You can rewrite the key. Don't forget the quotes
    SECRET_KEY = "#VERY_HARD_KEY#"

    # The line below keeps the user list.
    USER_CREDS = os.path.expanduser("~/gimmethat_userlist.json")

    TITLE = ''

    # Biggest file size in bytes.
    MAX_CONTENT_LENGTH = None

    # Antivirus enabled
    SCAN = False

    # Port
    PORT = 5000



from subprocess import run


class NotificationCenter(object):
    @staticmethod
    def file_rejection(user, filepath_list):
        notification_header = "File rejection\n"
        notification_file_list = '<b>' + '</b><b>'.join(filepath_list) + '</b>'
        run(['notify-send',
            user + 'sent you some files but some of them were infected.\n\n"',
            'They are removed from file system'], shell=True)



import netifaces as ni


def get_ips():
    interfaces = ni.interfaces()
    ips = []
    for interface in interfaces:
        try:
            if 'broadcast' in ni.ifaddresses(interface)[ni.AF_INET][0]:
                ips.append(ni.ifaddresses(interface)[ni.AF_INET][0]['addr'])
        except KeyError:
            pass
    return ips



from gimmethat.config import DefaultConfiguration
import json
import os


def init_users():
    if not os.path.exists(os.path.expanduser(DefaultConfiguration.UPLOAD_DIR)):
        print('Creating upload directory: {}'.format(
            os.path.expanduser(DefaultConfiguration.UPLOAD_DIR)))
        os.makedirs(os.path.expanduser(DefaultConfiguration.UPLOAD_DIR))

    if not os.path.exists(DefaultConfiguration.USER_CREDS):
        print('Creating credential list : {}'.format(DefaultConfiguration.USER_CREDS))
        save_creds()


def save_creds(creds=None):
    if not creds:
        creds = []
    with open(DefaultConfiguration.USER_CREDS, 'w') as f:
        json.dump(creds, f, indent=4)


def load_creds():
    try:
        with open(DefaultConfiguration.USER_CREDS) as f:
            creds = json.load(f)
        return creds
    except Exception:
        save_creds()
    return []


def check_auth(username, password):
    """This function is called to check if a username /
    password combination is valid.
    """
    users = load_creds()
    for u in users:
        if u['username'] == username and u['password'] == password:
            return True



from flask import (Flask, request, url_for, Response,
                   abort, render_template, redirect)
from flask_bootstrap import Bootstrap
from werkzeug.utils import secure_filename
from gimmethat.config import DefaultConfiguration
from gimmethat.users import check_auth, init_users
from gimmethat.helpers import get_ips
from functools import wraps, partial
from datetime import datetime
import werkzeug
import os
from gimmethat.antivirus import scan_file
# from ipdb import set_trace


class ApplicationObject(Flask):
    """docstring for ApplicationObject"""
    def __init__(self, *args, **kwargs):
        super(ApplicationObject, self).__init__(*args, **kwargs)
        self.args = args
        self.kwargs = kwargs

    def start_service(self):
        # Console messages
        init_users()
        print('Running with following configuration:')
        if isinstance(self.config['MAX_CONTENT_LENGTH'], int):
            print('Max upload size: {} bytes'.format(
                self.config['MAX_CONTENT_LENGTH']))
        else:
            print('Max upload size: None')
        print('Scan uploaded files:', self.config['SCAN'])

        print('You can use the addresses below')
        for ip in get_ips():
            print('\thttp://{}:{}'.format(ip, self.config['PORT']))

        app.run(host="0.0.0.0", port=app.config['PORT'],
                debug=app.config['DEBUG'], threaded=True)


app = ApplicationObject(__name__)
app.config.from_object(DefaultConfiguration)
Bootstrap(app)


def requires_auth(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        auth = request.authorization
        if not auth or not check_auth(auth.username, auth.password):
            return Response(
                render_template('unauthorized.html'), 401,
                {'WWW-Authenticate': 'Basic realm="Login Required"'})
        return f(data={'username': auth.username,
                       'time': datetime.now()}, *args, **kwargs)
    return decorated


def file_stream_saver(total_content_length, content_type, filename,
                      content_length=None, timestamp='NONE'):
    if app.config['MAX_CONTENT_LENGTH']:
        if total_content_length > app.config['MAX_CONTENT_LENGTH']:
            return abort(413)
    filename = secure_filename(filename)
    file_upl_dir = os.path.join(app.config['UPLOAD_DIR'],
                                request.authorization.username,
                                timestamp)
    if not os.path.exists(file_upl_dir):
        os.makedirs(file_upl_dir)
    filepath = os.path.join(file_upl_dir, filename)
    fp = open(filepath, 'wb')
    return fp


@app.route('/')
@requires_auth
def index(data):
    return render_template('index.html', title=app.config['TITLE'])


@app.route('/info')
def info():
    return render_template('info.html')


@app.route('/', methods=['POST'])
@requires_auth
def upload(data):
    folder_name = data['username']
    if folder_name not in os.listdir(app.config['UPLOAD_DIR']):
        os.makedirs(os.path.join(app.config['UPLOAD_DIR'], folder_name))
    print('RECEIVED from "{}"'.format(folder_name))
    timestamp = str(datetime.now()).replace(':', '-')
    stream, form, files = werkzeug.formparser.parse_form_data(
        request.environ,
        stream_factory=partial(
            file_stream_saver,
            timestamp=timestamp))
    for name, fp in files.items():
        fp.close()
        if app.config['SCAN']:
            abs_filepath = os.path.join(
                app.config['UPLOAD_DIR'], folder_name, timestamp, name)
            scan_results = scan_file(abs_filepath)
            r = scan_results[abs_filepath]
            if r[0] == 'OK':
                print('\t"{}" => {}'.format(fp.filename, r[0]))
            elif r[0] == 'FOUND':
                print('\t"{}" => {} [{}] REMOVED!'.format(
                    fp.filename, r[0], r[1]))
                os.remove(abs_filepath)
            else:
                print('\t"{}" has an interesting scan result. Check below:')
                print(scan_results)
                print('It can be a configuration/permission error',
                      'if you are encountering interesting logs for every',
                      'file.')
        else:
            print('\t"{}"'.format(fp.filename))

    return redirect(url_for('success'))


@app.route('/success', methods=['GET'])
def success():
    return render_template('success.html')
